# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

from enum import IntEnum
from ...protocol_enum_meta import ProtocolEnumMeta

class MapType(IntEnum, metaclass=ProtocolEnumMeta):
    Normal = 0
    Pk = 3
