"""
EO protocol data structures.

See Also:
  - [eolib.protocol._generated][]
"""

from .serialization_error import *

from .map import *
from .net import *
from .pub import *

from ._generated import *
