# Generated from the eo-protocol XML specification.
#
# This file should not be modified.
# Changes will be lost when code is regenerated.

"""
Data structures generated from the [eo-protocol](https://github.com/cirras/eo-protocol/tree/master/xml/net/protocol.xml){target="_blank"} XML specification.

Warning:
  - This subpackage should not be directly imported. 
  - Instead, import [eolib.protocol.net][] (or the top-level `eolib`).
"""

from .weight import *
from .version import *
from .three_item import *
from .spell import *
from .quest_page import *
from .party_request_type import *
from .packet_family import *
from .packet_action import *
from .item import *
from .char_item import *
